package com.persagy.dmp.rwd.define.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.map.MapUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.persagy.dmp.common.constant.ValidEnum;
import com.persagy.dmp.common.model.entity.BaseEntity;
import com.persagy.dmp.define.entity.ObjectInfoCollect;
import com.persagy.dmp.rwd.define.dao.ObjectInfoCollectMapper;
import com.persagy.dmp.rwd.define.service.IObjectInfoCollectService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * 平台、项目级信息点采集
 * @author:linhuili
 * @date:2021/10/13
 */
@Service
public class ObjectInfoCollectServiceImpl extends ServiceImpl<ObjectInfoCollectMapper,ObjectInfoCollect> implements IObjectInfoCollectService {

    @Autowired
    private ObjectInfoCollectMapper objectInfoCollectMapper;

    /**
     * 数量统计
     * @param queryWrapper 查询条件
     * @return
     */
    @Override
    public Integer queryCount(Wrapper<ObjectInfoCollect> queryWrapper) {
        return objectInfoCollectMapper.selectCount(queryWrapper);
    }

    /**
     * 查询采集的信息点
     * @param page 分页条件
     * @param queryWrapper 查询条件
     * @return
     */
    @Override
    public Page queryByCondition(Page page, Wrapper<ObjectInfoCollect> queryWrapper) {
        return objectInfoCollectMapper.selectPage(page, queryWrapper);
    }

    /**
     * 新增采集的信息点
     * @param voList 待新增数据
     * @return
     */
    @Override
    public List<ObjectInfoCollect> insert(List<ObjectInfoCollect> voList) throws Exception{
        saveOrUpdateBatch(voList);
        return voList;
    }

    /**
     * 修改采集的信息点
     * @param voList 待修改数据
     * @return
     */
    @Override
    public List<ObjectInfoCollect> update(List<ObjectInfoCollect> voList) throws Exception {
        saveOrUpdateBatch(voList);
        return voList;
    }

    /**
     * 删除采集的信息点
     * @param idList 主键
     */
    @Override
    public void delete(List<String> idList) {
        if(CollUtil.isEmpty(idList)) {
            return;
        }
        List<ObjectInfoCollect> dbList = objectInfoCollectMapper.selectBatchIds(idList);
        Map<String, ObjectInfoCollect> dbMap = CollectionUtil.fieldValueMap(dbList, BaseEntity.PROP_ID);
        idList.forEach(id -> {
            ObjectInfoCollect dbVO = MapUtil.get(dbMap, id, ObjectInfoCollect.class);
            // 容错处理
            if(dbVO == null) {
                return;
            }
            // 删除
            ObjectInfoCollect vo = new ObjectInfoCollect();
            vo.setId(id);
            vo.setValid(ValidEnum.FALSE.getType());
            objectInfoCollectMapper.updateById(vo);
        });
    }

}
